/************************************************************************
*  SimpleTestOpt.C (Simple Test Program for Minimization Algorithm)
*
*  Version:                     1.4
*  Date:                        11/30/94
*  Author:                      O. Knueppel
*************************************************************************/

//
// This test programs uses automatic differentiation to
// compute the derivatives of the function.
// It is a simple example for the easy-to-use driver "SimpleOptDriver".
//

#include "soptdrv.h"
#include "func.h"

//
// The Branin function is used as test function.
// This function has 5 global minima.
//

INTERVAL_AUTODIFF TestFunction (INTERVAL_VECTOR & v)
{
  INTERVAL_AUTODIFF x(v); /* Define independent variables */

  return Sqr (1.0 - 2.0 * x(2) + 0.05 * Sin (4.0 * Constant::Pi * x(2)) - x(1)) 
       + Sqr (x(2) - 0.5 * Sin (2.0 * Constant::Pi*x(1)));
}

VOID SetTestDomain (INTERVAL_VECTOR & x)
// The domain of the test function
{
  Resize (x, 2);
  x(1) = Hull (-10.0, 10.0);
  x(2) = Hull (-10.0, 10.0);
}


INT main()
{
  INT Iterations, BranchLevels;
  SOLUTIONLIST SolutionList;
  APPROXIMATIONLIST ApproximationList;
  INTERVAL_VECTOR TestDomain;
  INTERVAL ValueBounds;

  SetTestDomain (TestDomain);

  cout << "Computing all global minima of the Branin function" << endl << endl;

  cout << "To get all minima, use e.g. nit = 6, nd = 3" << endl << endl;

  cout << "nit = "; cin >> Iterations;
  cout << "nd  = "; cin >> BranchLevels;

  SimpleOptDriver (Iterations, BranchLevels, TestFunction, TestDomain,
		   SolutionList, ApproximationList, ValueBounds);

  cout << "f min in " << ValueBounds << endl;
  cout << "Solution List:" << endl << SolutionList << endl;
  cout << "Approximation List:" << endl << ApproximationList << endl;
  return 0;
}
