/************************************************************************
*  AppList.C (Approximation List Implementations)
*
*  Version:                     1.3
*  Date:                        11/17/95
*  Author:                      O. Knueppel
*************************************************************************/

static char sccs_id[] = "@(#)AppList.C 1.3 95/11/17 OK";

#include "applist.h"
#include "const.h"
#include "util.h"
#include "linlist.cge"
#undef LISTOBJECT
#undef LIST
#undef LIST_ELEMENT
#undef LISTCMPFUNC

APPROX_ELEMENT::APPROX_ELEMENT (REAL & va, INTERVAL_VECTOR & ra, VECTOR & p)
{
  MinValue = va; HeuristicBox = ra; MinPoint = p;
  Resize (MinEnclosure, 0);
  Resize (ExpansionBox, 0);
}

APPROX_ELEMENT::APPROX_ELEMENT(REAL & va, INTERVAL_VECTOR & ra,
			       INTERVAL_VECTOR & bo, INTERVAL_VECTOR & ebo,
			       INTERVAL& fbo, VECTOR & p)
{
  MinValue = va; HeuristicBox = ra; MinPoint = p;
  MinEnclosure = bo; ExpansionBox = ebo;
  MinValueEnclosure = fbo;
}

APPROX_ELEMENT & APPROX_ELEMENT::operator = (APPROX_ELEMENT & v)
{
  MinValue = v.MinValue; HeuristicBox = v.HeuristicBox;
  MinPoint = v.MinPoint;
  MinEnclosure = v.MinEnclosure; ExpansionBox = v.ExpansionBox;
  MinValueEnclosure = v.MinValueEnclosure;
  return *this;
}

ostream & operator << (ostream & o, APPROX_ELEMENT & v)
{
  o << endl << "    f(" << v.MinPoint << ") = " << v.MinValue << " over " << v.HeuristicBox;
  if (HasInclusion (v))
    {
      o << endl << "    x in:  " << v.MinEnclosure << endl;
      o << "    f in:  " << v.MinValueEnclosure << endl;
      o << "    x exp: " << v.ExpansionBox;
    }
  return o;
}

VOID Cleanup (APPROXIMATIONLIST & list, REAL threshold)
// Remove all entries with value > threshold
{
  if (IsEmpty (list)) return;
  First (list);
  while (!Finished (list))
    {
      if (Current (list).MinValue > threshold) RemoveCurrent (list);
      else Next (list);
    }
}

REAL Lowest (APPROXIMATIONLIST & list)
// Return lowest "MinValue" of all list elements
{
  REAL cur_min = Machine::PosInfinity;
  REAL t;

  First (list);
  while (!Finished (list))
    {
      t = Current (list).MinValue;
      if (t < cur_min)cur_min = t;
      Next (list);
    }
  return cur_min;
}

VOID Append (APPROXIMATIONLIST & list, VECTOR & vec, REAL val,
	     INTERVAL_VECTOR & ivec)
{
  APPROX_ELEMENT a(val, ivec, vec);

  list += a;
}

VOID Append (APPROXIMATIONLIST & list, VECTOR & vec, REAL val,
	     INTERVAL_VECTOR & ivec, INTERVAL_VECTOR & box,
	     INTERVAL_VECTOR & e_box, INTERVAL & fbox)
{
  APPROX_ELEMENT a(val, ivec, box, e_box, fbox, vec);

  list += a;
}

VOID Insert (APPROXIMATIONLIST & list, VECTOR & vec, REAL val,
	     INTERVAL_VECTOR & ivec)
{
  APPROX_ELEMENT a(val, ivec, vec);

  list <<= a;
}

VOID Append_First (APPROXIMATIONLIST & list, VECTOR & vec, REAL val,
		   INTERVAL_VECTOR & ivec)
{
  APPROX_ELEMENT a(val, ivec, vec);

  list *= a;
}

VOID Append (APPROXIMATIONLIST & dest, APPROXIMATIONLIST & source)
{
  while (!IsEmpty (source)) {
    dest += First (source);
    --source;
  }
}

BOOL Approximation_Exists (APPROXIMATIONLIST & list, INTERVAL_VECTOR & v)
{
  if (Length (list) != 0)
    for (First (list); !Finished (list); Next (list))
      if (Current (list).MinPoint <= v) return TRUE;
  return FALSE;
}

BOOL Approximation_Exists (APPROXIMATIONLIST & list, INTERVAL_VECTOR & v, VECTOR & app)
{
  if (Length (list) != 0)
    for (First (list); !Finished (list); Next (list))
      if (Current (list).MinPoint <= v)
	{
	  app = Current (list).MinPoint;
	  return TRUE;
	}
  return FALSE;
}

BOOL Expansion_Exists (APPROXIMATIONLIST & list, INTERVAL_VECTOR & v)
{
  if (Length (list) != 0)
    for (First (list); !Finished (list); Next (list))
      if (HasInclusion (Current (list)) && (v <= Current (list).ExpansionBox))
	return TRUE;
  return FALSE;
}

BOOL Expansion_Exists (APPROXIMATIONLIST & list, VECTOR & v)
{
  if (Length (list) != 0)
    for (First (list); !Finished (list); Next (list))
      if (HasInclusion (Current (list)) && (v <= Current (list).ExpansionBox))
	return TRUE;
  return FALSE;
}

BOOL Approximation_In_Range (APPROXIMATIONLIST & list, INTERVAL_VECTOR & v)
{
  INTERVAL_VECTOR dummy (Dimension (v));

  if (Length (list) != 0)
    for (First (list); !Finished (list); Next (list)) {
      if (HasInclusion (Current (list))
	  && Intersection (dummy, v, Current (list).ExpansionBox))
	return TRUE;
      if (Intersection (dummy, v, Current (list).HeuristicBox)) return TRUE;
    }
  return FALSE;
}
