/************************************************************************
*  AppList.h (Approximation List Definitions)
*
*  Version:                     1.1
*  Date:                        2/17/94
*  Author:                      O. Knueppel
*************************************************************************/

#ifndef __APPLIST__
#define __APPLIST__

#include "ivalmat.h"
#include "boolean.h"

class APPROX_ELEMENT {
 public:
  REAL            MinValue;            // approx. minimum value
  VECTOR          MinPoint;            // approx. minimum point
  INTERVAL_VECTOR HeuristicBox;        // heuristic area around minimum point
  INTERVAL_VECTOR MinEnclosure;        // enclosure of minimum point
  INTERVAL        MinValueEnclosure;   // enclosure of minimum value
  INTERVAL_VECTOR ExpansionBox;        // expansion around minimum point
  APPROX_ELEMENT() {};
  APPROX_ELEMENT(REAL &, INTERVAL_VECTOR &, VECTOR &);
  APPROX_ELEMENT(REAL &, INTERVAL_VECTOR &, INTERVAL_VECTOR &,
		 INTERVAL_VECTOR &, INTERVAL &, VECTOR &);
  APPROX_ELEMENT & operator = (APPROX_ELEMENT &);
  friend BOOL HasInclusion (APPROX_ELEMENT & a) { return Dimension (a.MinEnclosure) != 0; }
  friend ostream & operator << (ostream &, APPROX_ELEMENT &);
};

#undef LISTOBJECT
#undef LIST
#undef LIST_ELEMENT
#undef LISTCMPFUNC
#define LISTCMPFUNC  APPROXCMPFUNC
#define LIST_ELEMENT APPROX_ELEMENT
#define LIST         APPROXIMATIONLIST
#define LISTOBJECT   APPROXLISTOBJECT
#include "linlist.hge"

VOID Cleanup (APPROXIMATIONLIST &, REAL);
REAL Lowest (APPROXIMATIONLIST &);

VOID Append (APPROXIMATIONLIST &, VECTOR &, REAL, INTERVAL_VECTOR &);
VOID Append (APPROXIMATIONLIST &, VECTOR &, REAL, INTERVAL_VECTOR &,
	     INTERVAL_VECTOR &, INTERVAL_VECTOR &, INTERVAL &);
VOID Insert (APPROXIMATIONLIST &, VECTOR &, REAL, INTERVAL_VECTOR &);
VOID Append_First (APPROXIMATIONLIST &, VECTOR &, REAL, INTERVAL_VECTOR &);

VOID Append (APPROXIMATIONLIST &, APPROXIMATIONLIST &);

BOOL Approximation_Exists (APPROXIMATIONLIST &, INTERVAL_VECTOR &);
BOOL Approximation_Exists (APPROXIMATIONLIST &, INTERVAL_VECTOR &, VECTOR &);

BOOL Expansion_Exists (APPROXIMATIONLIST &, INTERVAL_VECTOR &);
BOOL Expansion_Exists (APPROXIMATIONLIST &, VECTOR &);

BOOL Approximation_In_Range (APPROXIMATIONLIST &, INTERVAL_VECTOR &);

#endif /* __APPLIST__ */
