/************************************************************************
*  Expand.C (Expansion Strategy Implementation)
*
*  Version:                     1.2
*  Date:                        2/21/94
*  Author:                      O. Knueppel
*************************************************************************/

static char sccs_id[] = "@(#)Expand.C 1.2 94/02/21 OK";

#include "expand.h"
#include "util.h"
#include "func.h"
#include "lss.h"
#include "thefunc.h"

#define Z_SIGMA z = Sup ((Abs (-R * g) + Abs (Id (dim) - R * h) * Hull (sigma * w)));

BOOL ExpansionStep (VECTOR & x, INTERVAL_VECTOR & Enclosure,
		    INTERVAL_VECTOR & Expansion)
{
  INT dim = Dimension (x);
  INT j;
  REAL tau = 0.001;
  REAL sigma = 0.00001;
  REAL expansion = 2;
  VECTOR w(dim), z(dim);
  INTERVAL_VECTOR g(dim);
  MATRIX R(dim, dim);
  INTERVAL_MATRIX h(dim, dim);
  INTERVAL_VECTOR x_dummy (dim), y_dummy;
  INT dummy;
  BOOL incl;
  
  for (j = 1; j <= dim; j++)
    if ((w(j) = Abs(x(j))) < tau) w(j) = 1.0;

  g = IntervalGradient (Hull (x));
  h = IntervalHessian (x + sigma * SymHull(w));
  ErrorHandler::SevereLevel = ErrorHandler::SevereError + 2;
  R = Inverse (Mid (h));
  ErrorHandler::SevereLevel = ErrorHandler::SevereError;
  Z_SIGMA;
  if (!(z < sigma * w)) return FALSE;
  Enclosure = x + SymHull (z);
  sigma = 0.1;
  h = IntervalHessian (x + sigma * SymHull(w));
  Z_SIGMA;
  if ((z < sigma * w)) {
    do {
      Expansion = x + SymHull (sigma * w);
      sigma *= expansion;
      h = IntervalHessian (x + sigma * SymHull(w));
      Z_SIGMA;
    } while ((z < sigma * w) && !(FunctionDomain <= Expansion));
  }
  else {
    do {
      sigma /= expansion;
      h = IntervalHessian (x + sigma * SymHull(w));
      Z_SIGMA;
      Expansion = x + SymHull (sigma * w);
    } while (!(z < sigma * w) && !(Expansion <= Enclosure));
  }
  if ((Expansion <= Enclosure)) Expansion = Enclosure;

  g = x - ILSS (IntervalHessian (Enclosure), g, incl);
  x_dummy = Enclosure;
  if (incl) Intersection (Enclosure, g, x_dummy);
  return TRUE;
}
