/************************************************************************
*  VecList.C (Solution List Implementations)
*
*  Version:                     1.1
*  Date:                        2/17/94
*  Author:                      O. Knueppel
*************************************************************************/

static char sccs_id[] = "@(#)VecList.C 1.1 94/02/17 OK";

#include "veclist.h"
#include "const.h"
#include "linlist.cge"
#undef LISTOBJECT
#undef LIST
#undef LIST_ELEMENT
#undef LISTCMPFUNC

SOLUTIONLIST_ELEMENT::SOLUTIONLIST_ELEMENT (INTERVAL & va, INTERVAL_VECTOR & ve)
{
  FunctionValue = va;
  Box = ve;
}

SOLUTIONLIST_ELEMENT & SOLUTIONLIST_ELEMENT::operator = (SOLUTIONLIST_ELEMENT & v)
{
  FunctionValue = v.FunctionValue;
  Box = v.Box;
  return *this;
}

ostream & operator << (ostream & o, SOLUTIONLIST_ELEMENT & v)
{
  return o << v.Box << "->" << v.FunctionValue;
}

BOOL SolutionCmp (SOLUTIONLIST_ELEMENT & obj1, SOLUTIONLIST_ELEMENT & obj2)
// Compare subroutine for sorting the list by increasing lower bounds of FunctionValue
{
  return (Inf (obj1.FunctionValue) <= Inf (obj2.FunctionValue));
}

VOID Cleanup (SOLUTIONLIST & list, REAL threshold)
// Delete all list entries with Inf(F(y)) > threshold.
{
  if (IsEmpty (list)) return;
  First (list);
  while (!Finished (list))
    {
      if (Inf (Current (list).FunctionValue) > threshold) RemoveCurrent (list);
      else Next (list);
    }
}

REAL Lowest(SOLUTIONLIST& list)
// return Min{Inf(F(y))}
{
  REAL cur_min = Machine::PosInfinity;
  REAL inf;

  First (list);
  while (!Finished (list))
    {
      inf = Inf (Current (list).FunctionValue);
      if (inf < cur_min) cur_min = inf;
      Next (list);
    }
  return cur_min;
}

VOID Append (SOLUTIONLIST & list, INTERVAL_VECTOR & vec, INTERVAL & value)
{
  SOLUTIONLIST_ELEMENT iv(value, vec);
  list += iv;
}

VOID Insert (SOLUTIONLIST & list, INTERVAL_VECTOR & vec, INTERVAL & value)
{
  SOLUTIONLIST_ELEMENT iv(value, vec);
  list <<= iv;
}

VOID Append_First (SOLUTIONLIST & list, INTERVAL_VECTOR & vec, INTERVAL & value)
{
  SOLUTIONLIST_ELEMENT iv(value, vec);
  list *= iv;
}

VOID Append (SOLUTIONLIST & dest, SOLUTIONLIST & source)
{
  while (!IsEmpty (source)) {
    dest += First (source);
    source--;
  }
}
